@@ -67,9 +67,10 @@ extern crate rustfft;
 #[cfg(any(feature = "doc", feature="use_gpu"))]
 extern crate clfft;
 extern crate arrayvec;
+#[macro_use]
+mod simd_extensions;
 mod vector_types;
 mod multicore_support;
-mod simd_extensions;
 pub mod window_functions;
 pub mod conv_types;
 pub use vector_types::*;
@@ -89,9 +90,11 @@ pub mod numbers {
     use std::fmt::Debug;
     use rustfft;
     use num_traits;
+    use simd_extensions;
     use simd_extensions::*;
     use gpu_support::{Gpu32, Gpu64, GpuRegTrait, GpuFloat};
-    use std::ops::*;
+    #[cfg(any(feature="use_sse", feature="use_avx", feature="use_avx512"))]
+    use simd;
 
     /// A trait for a numeric value which at least supports a subset of the operations defined in this crate.
     /// Can be an integer or a floating point number. In order to have support for all operations in this crate
@@ -108,19 +111,54 @@ pub mod numbers {
     /// Associates a number type with a SIMD register type.
     pub trait ToSimd: Sized + Sync + Send {
         /// Type for the SIMD register on the CPU.
-        type Reg: Simd<Self> + SimdGeneric<Self> + SimdApproximations<Self> + Copy + Sync + Send + Add<Output = Self::Reg> + Sub<Output = Self::Reg> + Mul<Output = Self::Reg> + Div<Output = Self::Reg> + Zero;
+        type RegFallback: SimdGeneric<Self>;
+        type RegSse : SimdGeneric<Self>;
+        type RegAvx : SimdGeneric<Self>;
+        type RegAvx512 : SimdGeneric<Self>;
         /// Type for the SIMD register on the GPU. Defaults to an arbitrary type if GPU support is not
         /// compiled in.
         type GpuReg: GpuRegTrait;
     }
 
     impl ToSimd for f32 {
-        type Reg = Reg32;
+        type RegFallback = simd_extensions::fallback::f32x4;
+        
+        #[cfg(feature="use_sse")]
+        type RegSse = simd::f32x4;
+        #[cfg(not(feature="use_sse"))]
+        type RegSse = simd_extensions::fallback::f32x4;
+        
+        #[cfg(feature="use_avx")]
+        type RegAvx = simd::x86::avx::f32x8;
+        #[cfg(not(feature="use_avx"))]
+        type RegAvx = simd_extensions::fallback::f32x4;
+        
+        #[cfg(feature="use_avx512")]
+        type RegAvx512 = stdsimd::simd::f32x16; // Type is missing in SIMD
+        #[cfg(not(feature="use_avx512"))]
+        type RegAvx512 = simd_extensions::fallback::f32x4;
+        
         type GpuReg = Gpu32;
     }
 
     impl ToSimd for f64 {
-        type Reg = Reg64;
+        type RegFallback = simd_extensions::fallback::f64x2;
+        
+        #[cfg(feature="use_sse")]
+        type RegSse = simd::x86::sse2::f64x2;
+        #[cfg(not(feature="use_sse"))]
+        type RegSse = simd_extensions::fallback::f64x2;
+        
+        #[cfg(feature="use_avx")]
+        type RegAvx = simd::x86::avx::f64x4;
+        #[cfg(not(feature="use_avx"))]
+        type RegAvx = simd_extensions::fallback::f64x2;
+        
+        #[cfg(feature="use_avx512")]
+        type RegAvx512 = stdsimd::simd::f64x8; // Type is missing in SIMD
+        #[cfg(not(feature="use_avx512"))]
+        type RegAvx512 = simd_extensions::fallback::f64x2;
+        
         type GpuReg = Gpu64;
     }
 
@@ -210,7 +248,7 @@ mod tests {
         // This is more a check for syntax. So if it compiles
         // then the test already passes. The assert is then only
         // a sanity check.
-        let reg = <f32 as ToSimd>::Reg::splat(1.0);
+        let reg = <f32 as ToSimd>::RegFallback::splat(1.0);
         let sum = reg.sum_real();
         assert!(sum > 0.0);
     }
