@@ -22,16 +22,22 @@
 // (this is the zlib license)
 //
 
-use super::{Simd, SimdApproximations, Reg32, IntReg32, UIntReg32, Reg64, IntReg64, UIntReg64, SimdFrom};
+use super::{Simd, SimdApproximations, SimdFrom};
+#[cfg(feature="use_sse")]
+use simd::*;
+#[cfg(feature="use_sse")]
+use simd::x86::sse2::*;
+#[cfg(feature="use_avx")]
+use simd::x86::avx::*;
 use std::mem;
 use std::ops::*;
 use numbers::*;
 use Zero;
 
 macro_rules! simd_approx_impl {
-    ($data_type:ident,
-     $bit_len:expr,
-     $regf:ident,
+    ($data_type: ident,
+     $bit_len: expr,
+     $regf: ident,
      $regi: ident,
      $regu: ident)
     =>
@@ -72,7 +78,7 @@ macro_rules! simd_approx_impl {
                 let log_q2 = $regf::splat(0.693359375);
 
                 let invalid_mask = x.le($regf::zero());
-                let x = unsafe { x.max(mem::transmute(min_norm_pos)) }; // cut off denormalized stuff
+                let x = unsafe { Simd::<$data_type>::max(x, mem::transmute(min_norm_pos)) }; // cut off denormalized stuff
                 let x: $regi = unsafe { mem::transmute(x) };
                 let emm0 = x.shr(mant_len);
 
@@ -162,9 +168,8 @@ macro_rules! simd_approx_impl {
                 let exp_p4 = $regf::splat(1.6666665459E-1);
                 let exp_p5 = $regf::splat(5.0000001201E-1);
 
-
-                let x = x.min(exp_hi);
-                let x = x.max(exp_lo);
+				let x = unsafe { Simd::<$data_type>::min(x, mem::transmute(exp_hi)) }; 
+                let x = unsafe { Simd::<$data_type>::max(x, mem::transmute(exp_lo)) }; 
 
                 // express exp(x) as exp(g + n*log(2))
                 let fx = x * log2ef + half;
@@ -343,12 +348,21 @@ macro_rules! simd_approx_impl {
     }
 }
 
-simd_approx_impl!(f32, 32, Reg32, IntReg32, UIntReg32);
-simd_approx_impl!(f64, 64, Reg64, IntReg64, UIntReg64);
+#[cfg(feature="use_sse")]
+simd_approx_impl!(f32, 32, f32x4, i32x4, u32x4);
+#[cfg(feature="use_sse")]
+simd_approx_impl!(f64, 64, f64x2, i64x2, u64x2);
+#[cfg(feature="use_avx")]
+simd_approx_impl!(f32, 32, f32x8, i32x8, u32x8);
+#[cfg(feature="use_avx")]
+simd_approx_impl!(f64, 64, f64x4, i64x4, u64x4);
 
 #[cfg(test)]
+#[cfg(feature="use_sse")]
 mod tests {
     use super::super::*;
+    use simd::f32x4;
+	use simd::x86::sse2::f64x2;
     use RealNumber;
 
     fn assert_eq_tol<T>(left: T, right: T, tol: T)
@@ -366,7 +380,7 @@ mod tests {
     #[test]
     fn ln_approx_test5() {
         let value = 5.0;
-        let reg = Reg32::splat(value);
+        let reg = f32x4::splat(value);
         let res = reg.ln_approx();
         assert_eq_tol(res.extract(0), value.ln(), 1e-9);
     }
@@ -374,7 +388,7 @@ mod tests {
     #[test]
     fn ln_approx_test1e8() {
         let value = 1e8;
-        let reg = Reg32::splat(value);
+        let reg = f32x4::splat(value);
         let res = reg.ln_approx();
         assert_eq_tol(res.extract(0), value.ln(), 1e-9);
     }
@@ -382,21 +396,21 @@ mod tests {
     #[test]
     fn ln_approx_test_small_value() {
         let value = 1e-8;
-        let reg = Reg32::splat(value);
+        let reg = f32x4::splat(value);
         let res = reg.ln_approx();
         assert_eq_tol(res.extract(0), value.ln(), 1e-9);
     }
 
     #[test]
     fn ln_approx_test_zero() {
-        let reg = Reg32::splat(0.0);
+        let reg = f32x4::splat(0.0);
         let res = reg.ln_approx();
         assert!(res.extract(0).is_nan());
     }
 
     #[test]
     fn ln_approx_test_neg() {
-        let reg = Reg32::splat(-5.0);
+        let reg = f32x4::splat(-5.0);
         let res = reg.ln_approx();
         assert!(res.extract(0).is_nan());
     }
@@ -404,7 +418,7 @@ mod tests {
     #[test]
     fn ln_approx_test_f64() {
         let value = 5.0;
-        let reg = Reg64::splat(value);
+        let reg = f64x2::splat(value);
         let res = reg.ln_approx();
         assert_eq_tol(res.extract(0), value.ln(), 1e-9);
     }
@@ -412,7 +426,7 @@ mod tests {
     #[test]
     fn exp_approx_test5() {
         let value = 5.0;
-        let reg = Reg32::splat(value);
+        let reg = f32x4::splat(value);
         let res = reg.exp_approx();
         assert_eq_tol(res.extract(0), value.exp(), 1e-9);
     }
@@ -420,7 +434,7 @@ mod tests {
     #[test]
     fn exp_approx_test_f64() {
         let value = 5.0;
-        let reg = Reg64::splat(value);
+        let reg = f64x2::splat(value);
         let res = reg.exp_approx();
         assert_eq_tol(res.extract(0), value.exp(), 1e-6);
     }
@@ -428,7 +442,7 @@ mod tests {
     #[test]
     fn sin_approx_test5() {
         let value = 5.0;
-        let reg = Reg32::splat(value);
+        let reg = f32x4::splat(value);
         let res = reg.sin_approx();
         assert_eq_tol(res.extract(0), value.sin(), 1e-9);
     }
@@ -436,7 +450,7 @@ mod tests {
     #[test]
     fn sin_approx_test_f64() {
         let value = 5.0;
-        let reg = Reg64::splat(value);
+        let reg = f64x2::splat(value);
         let res = reg.sin_approx();
         assert_eq_tol(res.extract(0), value.sin(), 1e-9);
     }
@@ -444,7 +458,7 @@ mod tests {
     #[test]
     fn cos_approx_test5() {
         let value = 5.0;
-        let reg = Reg32::splat(value);
+        let reg = f32x4::splat(value);
         let res = reg.cos_approx();
         assert_eq_tol(res.extract(0), value.cos(), 1e-7);
     }
@@ -452,7 +466,7 @@ mod tests {
     #[test]
     fn cos_approx_testf64() {
         let value = 5.0;
-        let reg = Reg64::splat(value);
+        let reg = f64x2::splat(value);
         let res = reg.cos_approx();
         assert_eq_tol(res.extract(0), value.cos(), 1e-7);
     }
