@@ -4,19 +4,7 @@ use simd::x86::avx::*;
 use std::arch::x86_64::*;
 use std::mem;
 
-pub type Reg32 = f32x8;
-
-pub type Reg64 = f64x4;
-
-pub type IntReg32 = i32x8;
-
-pub type IntReg64 = i64x4;
-
-pub type UIntReg32 = u32x8;
-
-pub type UIntReg64 = u64x4;
-
-/// This value must be read in groups of 3 bits.
+/// This value must be read in groups of 2 bits.
 const SWAP_IQ_PS: i32 = 0b10110001;
 
 /// This value must be read in groups of 2 bits:
