@@ -1,5 +1,11 @@
 use numbers::*;
 use std::mem;
+use std::ops::*;
+use std;
+#[cfg(feature="use_sse")]
+use simd::x86::sse2 as simdsse;
+#[cfg(feature="use_avx")]
+use simd::x86::avx as simdavx;
 
 /// SIMD methods which have `f32` or `f64` specific implementation.
 pub trait Simd<T>: Sized
@@ -13,9 +19,9 @@ pub trait Simd<T>: Sized
     
     /// The type of complex valued array which matches a SIMD register.
     type ComplexArray;
-	
-	/// Number of elements in a SIMD register.
-	const LEN: usize;
+    
+    /// Number of elements in a SIMD register.
+    const LEN: usize;
     
     /// Loads a SIMD register from an array. If the end of the array is approached
     /// then the load code wraps around to the beginning and starts to load the first 
@@ -86,7 +92,9 @@ pub trait SimdFrom<T> {
 }
 
 /// SIMD methods which share their implementation independent if it's a `f32` or `f64` register.
-pub trait SimdGeneric<T>: Simd<T>
+pub trait SimdGeneric<T>: Simd<T> + SimdApproximations<T>
+    + Add<Self, Output=Self> + Sub<Self, Output=Self> + Mul<Self, Output=Self> + Div<Self, Output=Self> 
+    + Copy + Clone + Sync + Send + Sized + Zero
     where T: Sized + Sync + Send
 {
     /// On some CPU architectures memory access needs to be aligned or otherwise
@@ -140,8 +148,7 @@ pub trait SimdGeneric<T>: Simd<T>
 /// The approximations are implemented based on SIMD registers.
 /// Refer to the documentation of the `ApproximatedOps` trait (which is part of 
 /// the public API of this lib) for some information about accuracy and speed.
-pub trait SimdApproximations<T>: Simd<T>
-    where T: Sized + Sync + Send
+pub trait SimdApproximations<T>
 {
     /// Returns the natural logarithm of the number.
     fn ln_approx(self) -> Self;
@@ -179,22 +186,22 @@ impl<T: Clone + Copy> Clone for Unalign<T> {
 } 
 
 macro_rules! simd_generic_impl {
-    ($data_type:ident, $reg:ident)
+    ($data_type:ident, $mod: ident::$reg:ident)
     =>
     {
-        impl Zero for $reg {
-            fn zero() -> $reg {
-                $reg::splat(0.0)
+        impl Zero for $mod::$reg {
+            fn zero() -> Self {
+                Self::splat(0.0)
             }
         }
 
-        impl SimdGeneric<$data_type> for $reg {
+        impl SimdGeneric<$data_type> for $mod::$reg {
             #[inline]
             fn calc_data_alignment_reqs(array: &[$data_type]) -> (usize, usize, usize) {
                 let data_length = array.len();
                 let addr = array.as_ptr();
                 let scalar_left = (addr as usize % mem::size_of::<Self>()) / mem::size_of::<f32>();
-                if scalar_left + $reg::LEN > data_length {
+                if scalar_left + Self::LEN > data_length {
                     // Result order: scalar_left, scalar_right, vectorization_length
                     (data_length, data_length, 0)
                 } else {
@@ -283,45 +290,87 @@ macro_rules! simd_generic_impl {
 
             #[inline]
             fn extract(self, idx: u32) -> $data_type {
-                $reg::extract(self, idx)
+                Self::extract(self, idx)
             }
 
             #[inline]
             fn splat(value: $data_type) -> Self {
-                $reg::splat(value)
+                Self::splat(value)
             }
 
             #[inline]
             fn store(self, array: &mut [$data_type], index: usize) {
-                $reg::store(self, array, index);
+                Self::store(self, array, index);
             }
         }
     }
 }
 
-#[cfg(any(feature = "doc", feature="use_avx"))]
+#[cfg(feature="use_avx512")]
+mod avx512;
+#[cfg(feature="use_avx512")]
+simd_generic_impl!(f32, simd::f32x16); // Type isn't implemented in simd
+#[cfg(feature="use_avx512")]
+simd_generic_impl!(f64, simd::f64x8);  // Type isn't implemented in simd
+
+#[cfg(feature="use_avx")]
 mod avx;
+#[cfg(feature="use_avx")]
+simd_generic_impl!(f32, simdavx::f32x8);
+#[cfg(feature="use_avx")]
+simd_generic_impl!(f64, simdavx::f64x4);
 
-#[cfg(any(feature = "doc", feature="use_avx"))]
-pub use self::avx::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
-
-#[cfg(any(feature = "doc", all(feature= "use_sse", not(feature= "use_avx"))))]
+#[cfg(feature="use_sse")]
 mod sse;
+#[cfg(feature="use_sse")]
+simd_generic_impl!(f32, simd::f32x4);
+#[cfg(feature="use_sse")]
+simd_generic_impl!(f64, simdsse::f64x2);
 
-#[cfg(any(feature = "doc", all(feature = "use_sse", not(feature = "use_avx"))))]
-pub use self::sse::{Reg32, Reg64, IntReg32, IntReg64, UIntReg32, UIntReg64};
-
-#[cfg(any(feature = "doc", any(feature = "use_sse", feature = "use_avx")))]
+#[cfg(any(feature="use_sse", feature="use_avx", feature="use_avx512"))]
 mod approximations;
 
-#[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
-mod fallback;
-
-#[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
-pub use self::fallback::{Reg32, Reg64};
-
-#[cfg(any(feature = "doc", not(any(feature = "use_avx", feature="use_sse"))))]
+pub mod fallback;
 mod approx_fallback;
 
-simd_generic_impl!(f32, Reg32);
-simd_generic_impl!(f64, Reg64);
+simd_generic_impl!(f32, fallback::f32x4);
+simd_generic_impl!(f64, fallback::f64x2);
+
+pub struct RegType<Reg> {
+    _type: std::marker::PhantomData<Reg>
+}
+
+impl<Reg> RegType<Reg> {
+    pub fn new() -> Self {
+        RegType { 
+            _type: std::marker::PhantomData 
+        }
+    }
+}
+
+/// Selects a SIMD register type and passes it as 2nd argument to a function.
+/// The macro tries to mimic the Rust syntax of a method call.
+macro_rules! sel_reg(
+    ($self_:ident.$method: ident::<$type: ident>($($args: expr),*)) => {
+        if is_x86_feature_detected!("avx512vl") && cfg!(feature="use_avx512") {
+            $self_.$method(RegType::<<$type as ToSimd>::RegAvx512>::new(), $($args),*)
+        } else if is_x86_feature_detected!("avx2") && cfg!(feature="use_avx") {
+            $self_.$method(RegType::<<$type as ToSimd>::RegAvx>::new(), $($args),*)
+        } else if is_x86_feature_detected!("sse2") && cfg!(feature="use_sse") {
+            $self_.$method(RegType::<<$type as ToSimd>::RegSse>::new(), $($args),*)
+        } else {
+            $self_.$method(RegType::<<$type as ToSimd>::RegFallback>::new(), $($args),*)
+        }
+    };
+    ($method: ident::<$type: ident>($($args: expr),*)) => {
+        if is_x86_feature_detected!("avx512vl") && cfg!(feature="use_avx512") {
+            $method(RegType::<<$type as ToSimd>::RegAvx512>::new(), $($args),*)
+        } else if is_x86_feature_detected!("avx2") && cfg!(feature="use_avx") {
+            $method(RegType::<<$type as ToSimd>::RegAvx>::new(), $($args),*)
+        } else if is_x86_feature_detected!("sse2") && cfg!(feature="use_sse") {
+            $method(RegType::<<$type as ToSimd>::RegSse>::new(), $($args),*)
+        } else {
+            $method(RegType::<<$type as ToSimd>::RegFallback>::new(), $($args),*)
+        }
+    };
+);
\ No newline at end of file
