@@ -10,23 +10,20 @@ mod dot_products;
 pub use self::dot_products::*;
 mod mapping;
 pub use self::mapping::*;
-#[cfg(feature="std")]
 mod statistics;
-#[cfg(feature="std")]
 pub use self::statistics::*;
-#[cfg(feature="std")]
 mod precise_stats;
-#[cfg(feature="std")]
 pub use self::precise_stats::*;
 
 use std::ops::{Add, Sub};
-use super::super::Zero;
+use numbers::*;
 
 /// Sums up the given values using a more precise
 /// summation algorithm: `https://en.wikipedia.org/wiki/Kahan_summation_algorithm`
 fn kahan_sum<I, T>(values: I) -> T
-    where I: Iterator<Item=T>,
-        T: Add<Output=T> + Sub<Output=T> + Zero + Copy {
+    where I: Iterator<Item = T>,
+          T: Add<Output = T> + Sub<Output = T> + Zero + Copy
+{
     let mut sum = T::zero();
     let mut c = T::zero();
     for n in values {
@@ -41,8 +38,9 @@ fn kahan_sum<I, T>(values: I) -> T
 /// Sums up the given values using a more precise
 /// summation algorithm: `https://en.wikipedia.org/wiki/Kahan_summation_algorithm`
 fn kahan_sumb<'a, I, T>(values: I) -> T
-    where I: Iterator<Item=&'a T>,
-        T: 'a + Add<Output=T> + Sub<Output=T> + Zero + Copy {
+    where I: Iterator<Item = &'a T>,
+          T: 'a + Add<Output = T> + Sub<Output = T> + Zero + Copy
+{
     let mut sum = T::zero();
     let mut c = T::zero();
     for n in values {
