@@ -69,9 +69,8 @@ pub trait ModuloOps<T>
 /// Recommended to be only used with the CPU feature flags `sse` or `avx`.
 ///
 /// This trait provides alternative implementations for some standard functions which
-/// are less accurate but perform faster. Those approximations are written for SSE2
-/// (target feature flag `sse`) or AVX2 (target feature flag `avx`) processors. Without any of those
-/// feature flags the standard library functions will be used instead.
+/// are less accurate but perform faster. Those approximations are written for SSE2 or AVX2 processors.
+/// If a processor supports neither of those instruction sets, then the standard functions will be used.
 ///
 /// Information on the error of the approximation and their performance are rough numbers.
 /// A detailed table can be obtained by running the `approx_accuracy` example.
@@ -240,10 +239,11 @@ impl<S, T, N, D> RealOps for DspVec<S, T, N, D>
 {
     fn abs(&mut self) {
         assert_real!(self);
-        self.simd_real_operation(|x, _arg| (x * x).sqrt(),
+        sel_reg!(self.simd_real_operation::<T>(
+                                 |x, _arg| (x * x).sqrt(),
                                  |x, _arg| x.abs(),
                                  (),
-                                 Complexity::Small);
+                                 Complexity::Small));
     }
 }
 
@@ -295,64 +295,60 @@ impl<S, T, N, D> ApproximatedOps<T> for DspVec<S, T, N, D>
 {
     fn ln_approx(&mut self) {
         assert_real!(self);
-        self.simd_real_operation(|x, _arg| x.ln_approx(),
+        sel_reg!(self.simd_real_operation::<T>(|x, _arg| x.ln_approx(),
                                  |x, _arg| x.ln(),
                                  (),
-                                 APPROX_COMPLEXITY);
+                                 APPROX_COMPLEXITY));
     }
 
     fn exp_approx(&mut self) {
         assert_real!(self);
-        self.simd_real_operation(|x, _arg| x.exp_approx(),
+        sel_reg!(self.simd_real_operation::<T>(|x, _arg| x.exp_approx(),
                                  |x, _arg| x.exp(),
                                  (),
-                                 APPROX_COMPLEXITY);
+                                 APPROX_COMPLEXITY));
     }
 
     fn sin_approx(&mut self) {
         assert_real!(self);
-        self.simd_real_operation(|x, _arg| x.sin_approx(),
+        sel_reg!(self.simd_real_operation::<T>(|x, _arg| x.sin_approx(),
                                  |x, _arg| x.sin(),
                                  (),
-                                 APPROX_COMPLEXITY);
+                                 APPROX_COMPLEXITY));
     }
 
     fn cos_approx(&mut self) {
         assert_real!(self);
-        self.simd_real_operation(|x, _arg| x.cos_approx(),
+        sel_reg!(self.simd_real_operation::<T>(|x, _arg| x.cos_approx(),
                                  |x, _arg| x.cos(),
                                  (),
-                                 APPROX_COMPLEXITY);
+                                 APPROX_COMPLEXITY));
     }
 
     fn log_approx(&mut self, base: T) {
         assert_real!(self);
-        let base_ln = T::Reg::splat(base.ln());
-        self.simd_real_operation(|x, b| x.ln_approx() / b,
+        sel_reg!(self.simd_real_operationf::<T>(|x, b| x.ln_approx() / b,
                                  |x, b| x.ln() / b.extract(0),
-                                 base_ln,
-                                 APPROX_COMPLEXITY);
+                                 base.ln(),
+                                 APPROX_COMPLEXITY));
     }
 
     fn expf_approx(&mut self, base: T) {
         assert_real!(self);
         // Transform base with:
         // x^y = e^(ln(x)*y)
-        let base_ln = T::Reg::splat(base.ln());
-        self.simd_real_operation(|y, x| (x * y).exp_approx(),
-                                 |y, x| (x.extract(0) * y).exp(),
-                                 base_ln,
-                                 APPROX_COMPLEXITY);
+        sel_reg!(self.simd_real_operationf::<T>(|y, x| (x * y).exp_approx(),
+                                |y, x| (x.extract(0) * y).exp(),
+                                base.ln(),
+                                APPROX_COMPLEXITY));
     }
 
     fn powf_approx(&mut self, exponent: T) {
         assert_real!(self);
         // Transform base with the same equation as for `expf_approx`
-        let exponent = T::Reg::splat(exponent);
-        self.simd_real_operation(|x, y| (x.ln_approx() * y).exp_approx(),
+        sel_reg!(self.simd_real_operationf::<T>(|x, y| (x.ln_approx() * y).exp_approx(),
                                  |x, y| (x.ln() * y.extract(0)).exp(),
                                  exponent,
-                                 APPROX_COMPLEXITY);
-
+                                 APPROX_COMPLEXITY));
     }
 }
